package com.boman.system.service;

import java.util.List;
import java.util.Map;
import java.util.Set;


import com.boman.common.core.web.domain.AjaxResult;
import com.boman.domain.SysMenu;
import com.boman.system.domain.vo.RouterVo;
import com.boman.system.domain.vo.TreeSelect;

/**
 * 菜单 业务层
 * 
 * @author ruoyi
 */
public interface ISysMenuService
{
    /**
     * 根据用户查询系统菜单列表
     * 
     * @param userId 用户ID
     * @return 菜单列表
     */
    public List<SysMenu> selectMenuList(Long userId);

    /**
     * 根据用户查询系统菜单列表
     * 
     * @param menu 菜单信息
     * @param userId 用户ID
     * @return 菜单列表
     */
    public List<SysMenu> selectMenuList(SysMenu menu, Long userId);

    public List<SysMenu> selectMenuListAll(SysMenu menu);

    /**
     * 功能描述: 获取menu下所有的子节点
     *
     * @param allMenus 所有的
     * @param menu     父节点吧
     * @return java.util.List<com.boman.domain.SysMenu>
     */
    List<SysMenu> getChildList(List<SysMenu> allMenus, SysMenu menu);

    /**
     * 根据用户ID查询权限
     * 
     * @param id 用户ID
     * @return 权限列表
     */
    public Set<String> selectMenuPermsById(Long id);

    /**
     * 根据用户ID查询菜单树信息
     * 
     * @param userId 用户ID
     * @return 菜单列表
     */
    public List<SysMenu> selectMenuTreeByUserId(Long userId);

    /**
     * 功能描述: 根据菜单id，找到此菜单以及子菜单下所有的叶子节点
     *
     * @param menuId menuId
     * @return java.util.List<com.boman.domain.SysMenu>
     */
    List<SysMenu> allLeafNodeById(Long menuId);

    /**
     * 根据角色ID查询菜单树信息
     * 
     * @param roleId 角色ID
     * @return 选中菜单列表
     */
    public List<Integer> selectMenuListByRoleId(Long roleId);

    /**
     * 构建前端路由所需要的菜单
     * 
     * @param menus 菜单列表
     * @return 路由列表
     */
    public List<RouterVo> buildMenus(List<SysMenu> menus);

    /**
     * 构建前端所需要树结构
     * 
     * @param menus 菜单列表
     * @return 树结构列表
     */
    public List<SysMenu> buildMenuTree(List<SysMenu> menus);
    /**
     * 构建前端所需要树结构 没有叶子结点
     *
     * @param menus 菜单列表
     * @return 树结构列表
     */
    public List<SysMenu> buildMenuTreeNotAddLeafNode(List<SysMenu> menus);

    /**
     * 构建前端所需要下拉树结构
     * 
     * @param menus 菜单列表
     * @return 下拉树结构列表
     */
    public List<TreeSelect> buildMenuTreeSelect(List<SysMenu> menus);

    /**
     * 构建前端所需要下拉树结构
     *
     * @param menus 菜单列表
     * @return 下拉树结构列表
     */
    public List<TreeSelect> buildMenuTreeSelectNotAddLeafNode(List<SysMenu> menus);

    /**
     * 根据菜单ID查询信息
     * 
     * @param id 菜单ID
     * @return 菜单信息
     */
    public SysMenu selectMenuById(Long id);

    /**
     * 是否存在菜单子节点
     * 
     * @param id 菜单ID
     * @return 结果 true 存在 false 不存在
     */
    public boolean hasChildById(Long id);

    /**
     * 查询菜单是否存在角色
     * 
     * @param id 菜单ID
     * @return 结果 true 存在 false 不存在
     */
    public boolean checkMenuExistRole(Long id);

    /**
     * 新增保存菜单信息
     * 
     * @param menu 菜单信息
     * @return 结果
     */
    public AjaxResult insertMenu(SysMenu menu);

    /**
     * 修改保存菜单信息
     * 
     * @param menu 菜单信息
     * @return 结果
     */
    public int updateMenu(SysMenu menu);

    /**
     * 删除菜单管理信息
     * 
     * @param id 菜单ID
     * @return 结果
     */
    public int deleteMenuById(Long id);

    public int removeMenuById(Long id);

    /**
     * 校验菜单名称是否唯一
     * 
     * @param menu 菜单信息
     * @return 结果
     */
    public String checkMenuNameUnique(SysMenu menu);

    Map<String, Object> listMenus(Long roleId, Long menuId);

    /**
     * 功能描述: 根据roleId查找的叶子结点去匹配所对应的菜单树
     *
     * @param roleId roleId
     * @return java.lang.String
     */
    List<SysMenu>  listTreeByRoleId(Long roleId);

    /**
     * 功能描述: 找到sonMenus对应的所有的父类，结果存放于result中
     *
     * @param allMenus    allMenus
     * @param targetMenus 就是查他的祖宗
     * @param parentList  targetMenus中所有的祖宗都在这里面
     */
    void recGetParent(List<SysMenu> allMenus, List<SysMenu> targetMenus, List<SysMenu> parentList);

    /**
     * 根据用户ID查询按钮
     *
     * @param userId 用户ID
     * @return 菜单列表
     */
    List<SysMenu> listBtnByUserId(Long userId);

}
